SUBROUTINE perm_wdh_init
  USE machine_constants, ONLY: kdp
  USE parameters, ONLY: wdh_on, icrxtype
  USE wdh
  USE mesh, ONLY: nxyz
  USE control, ONLY: maxdelt
  USE units
  IMPLICIT NONE
  integer ic
  REAL(KIND=kdp) :: perm_ref_msq, perm_ductile_msq, perm_min_msq, cohesive_strength_MPa, tensile_strength_MPa
  ! value of wdh_control     meaning
  !          -1               undefined
  !           0               temperature above T_solidus, perm at minimum
  !           1               pff <= 1 and k_T >= k_D, perm is k_D
  !           2               pff <= 1 and k_T < k_D, perm is between k_D and k_T
  !           3               pff > 1, perm is k_H2 = xk_old(ic) * pore_fluid_factor**2
  !           4               pff > 1 and k_H2 < k_D, perm is k_D
  !           5               pff > 1 and k_H2 > 100*k_D, perm is 100*K_D
  !           6               pff > 1 and k_H2 > k_ref, perm is k_ref 
  open (98, err=30, file='perm_wdh.in')
  read (98, *, err=30)
  read (98, *, err=30) wdh_on                       ! T = wdh model. F = run as standard hydrotherm
  if (.not.wdh_on) then
    wdh_explicit = .false.
    close(98)
    return
  end if
  rewind (98)
  read (98, *, err=30)
  read (98, *, err=30) wdh_on, wdh_explicit, maxdelt
  IF(iyr == 2) maxdelt = sec(maxdelt)
  read (98, *, err=30)
  read (98, *, err=30) log_perm_A                   ! -14 in WDH, log k = - log_perm_A - log_perm_B * log(z_km)
  read (98, *, err=30)
  read (98, *, err=30) log_perm_B                   ! -3.2 in WDH
  read (98, *, err=30)
  read (98, *, err=30) perm_ref_msq                 ! 1e-14 m^2 in  WDH
  read (98, *, err=30)
  read (98, *, err=30) perm_ductile_msq             ! 1e-17 m^2 in  WDH
  read (98, *, err=30)
  read (98, *, err=30) perm_min_msq                 ! 1e-22 m^2 in  WDH
  read (98, *, err=30)
  read (98, *, err=30) perm_anistropy               ! ratio of vertical to horizontal permeability
  read (98, *, err=30)
  read (98, *, err=30) cohesive_strength_MPa        ! 10 MPa in WDH
  read (98, *, err=30)
  read (98, *, err=30) T_solidus                    ! 700 in WDH
  read (98, *, err=30)
  read (98, *, err=30) T_ductile_max                ! 500 in WDH
  read (98, *, err=30)
  read (98, *, err=30) T_ductile_k_defined          ! 400 in WDH
  read (98, *, err=30)
  read (98, *, err=30) T_ductile_min                ! 360 in WDH
  close (98)
  LOG_K_REF     = log10(perm_ref_msq * 1.0e4_kdp)     ! permeability converted from m^2 to cm^2
  LOG_K_DUCTILE = log10(perm_ductile_msq * 1.0e4_kdp)
  LOG_K_MIN     = log10(perm_min_msq * 1.0e4_kdp)
  cohesive_strength = cohesive_strength_MPa * 1.0e7_kdp  ! cohesive strength converted from MPa to dyne/cm^2
  allocate (wdh_control(nxyz), wdh_control_old(nxyz))
  wdh_control = -1
  wdh_control_old = -1
  allocate (wdh_magma(nxyz))
  do ic = 1, nxyz
     if (icrxtype(ic) == 2) then  ! this assumes that rock type 2 indicates initially molten magma
        wdh_magma(ic) = .true.
     else
        wdh_magma(ic) = .false.
     end if
  end do
  return
  30 stop    ! error in reading input file
END SUBROUTINE perm_wdh_init

SUBROUTINE perm_wdh
  USE machine_constants, ONLY: kdp
  USE mesh
  USE parameters
  USE variables
  USE i_c
  USE wdh
  USE control, ONLY: ltime, nppn
  IMPLICIT NONE
  INTEGER :: ic, icdum, i, j, k
  DO  icdum = 1, npiccons
     ic = npic(icdum)
     CALL ictoijk(ic,i,j,k,nx,nz)
     ! Check if magma has crystallized
     if (nppn > 1 .and. wdh_magma(ic)) then
        if (Tc(ic) >= T_solidus) then
           ! magma has not crystallized
           xk(ic) = 10.0_kdp**LOG_K_MIN
           wdh_control(ic) = 0
           CYCLE
        else
           ! magma has crystallized
           wdh_magma(ic) = .false.
        end if
     end if
     ! compute depth dependent k
     ! Depth is computed from top of the grid
     depth_km = ((zz(nz)+0.5_kdp*dz(nz))-(zz(k)-0.5_kdp*dz(k))) * 1.0e-5_kdp
     log_k_D = (log_perm_A + log_perm_B*LOG10(depth_km)) + 4.0_kdp
     IF (log_k_D > -10.0_kdp) log_k_D = -10.0_kdp
     k_D = 10.0_kdp**log_k_D
     ! compute differential stress
     if (Tc(ic) >= T_ductile_max) then
       diff_stress = 0
     else
       ! Cox 
       diff_stress = cohesive_strength + (1.0_kdp - pinit(ic)/plith(ic))*3.0_kdp*plith(ic)/4.0_kdp
       if (diff_stress < 2.5_kdp*cohesive_strength) then
          diff_stress = 0.5_kdp*cohesive_strength + (1.0_kdp - pinit(ic)/plith(ic)) * plith(ic)
       end if
       diff_stress = diff_stress - 0.5_kdp*cohesive_strength
       if (Tc(ic) > T_ductile_min) then
          diff_stress = diff_stress * ((T_ductile_max - Tc(ic))/(T_ductile_max - T_ductile_min))**2
       end if
     end if
     ! compute failure pressure and pore fluid factor
     if (diff_stress < 2.5_kdp*cohesive_strength) then
       ! extension failure
       failure_pressure = 0.5_kdp * cohesive_strength + plith(ic) - diff_stress
     else
       ! brittle failure
       failure_pressure = (4.0_kdp*cohesive_strength - plith(ic) + 4.0_kdp*(plith(ic)-diff_stress)) / 3.0_kdp
     end if
     pore_fluid_factor = p(ic)/failure_pressure
     ! compute temperature_dependent k
     IF (Tc(ic) <= T_ductile_min) THEN
       log_k_T = LOG_K_REF
     ELSE IF (Tc(ic) <= T_ductile_k_defined) THEN
       log_k_T = (LOG_K_REF*(T_ductile_k_defined - Tc(ic)) + LOG_K_DUCTILE*(Tc(ic) - T_ductile_min)) / (T_ductile_k_defined - T_ductile_min)
     ELSE If (Tc(ic) <= T_ductile_max) THEN
       log_k_T = (LOG_K_DUCTILE*(T_ductile_max - Tc(ic)) + LOG_K_MIN*(Tc(ic) - T_ductile_k_defined)) / (T_ductile_max - T_ductile_k_defined)
     ELSE
       log_k_T = LOG_K_MIN
     END IF
     k_T = 10.0_kdp**log_k_T
     ! Determine permeability
     if (p(ic) <= failure_pressure) then
        ! no hydraulic fracture
        if (wdh_control_old(ic) >= 3) then
           ! if this cell hydrofractured in the last time step, set the permeability to depth dependent
           xk(ic) = k_D
           wdh_control(ic) = 1
        else if (k_T < k_D) then
           pffN = (max(pore_fluid_factor, 0.3_kdp) - 0.3_kdp) / 0.7_kdp
           log_k_Pff = log_k_D * pffN**2 + (1.0_kdp - pffN**2) * log_k_T
           xk(ic) = 10.0_kdp ** log_k_Pff
           wdh_control(ic) = 2
        else
           xk(ic) = k_D
           wdh_control(ic) = 1
        end if
     else
        !hydrofracture
        k_H2 = xk_old(ic) * pore_fluid_factor**2
        wdh_control(ic) = 3
        if (k_H2 < k_D) then
          k_H2 = k_D
          wdh_control(ic) = 4
        else if (k_H2 > k_D*100) then
          k_H2 = k_D*100.0_kdp
          wdh_control(ic) = 5
        end if
        if (k_H2 > 10.0_kdp**LOG_K_REF) then
          k_H2 = 10.0_kdp**LOG_K_REF
          wdh_control(ic) = 6
        end if
        xk(ic) = k_H2
     end if
  END DO
  DO  icdum = 1, npiccons
     ic = npic(icdum)
     yk(ic) = xk(ic)
     zk(ic) = perm_anistropy * xk(ic)
  END DO
END SUBROUTINE perm_wdh
